﻿Imports System
Imports System.Windows.Input

Public Class DelegateCommand(Of T)
    Implements ICommand
    Private ReadOnly _execute As Action(Of T)
    Private ReadOnly _canExecute As Predicate(Of T)

    Public Sub New(ByVal execute As Action(Of T))
        Me.New(execute, Function(x) True)
    End Sub

    Public Sub New(ByVal execute As Action(Of T), ByVal canExecute As Predicate(Of T))
        If canExecute Is Nothing Then
            Throw New ArgumentNullException("canExecute")
        End If
        If execute Is Nothing Then
            Throw New ArgumentNullException("execute")
        End If

        _execute = execute
        _canExecute = canExecute
    End Sub

    Public Sub Execute(ByVal parameter As Object) Implements ICommand.Execute
        _execute(CType(parameter, T))
    End Sub

    Public Function CanExecute(ByVal parameter As Object) As Boolean Implements ICommand.CanExecute
        Return _canExecute(CType(parameter, T))
    End Function

    Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

    Public Sub RaiseCanExecuteChanged()
        CanExecuteChangedEvent.Raise(Me)
    End Sub
End Class

Public Class DelegateCommand
    Inherits DelegateCommand(Of Object)
    Public Sub New(ByVal execute As Action)
        MyBase.New(If(execute IsNot Nothing, Function(x) execute, Nothing))
    End Sub

    Public Sub New(ByVal execute As Action, ByVal canExecute As Func(Of Boolean))
        MyBase.New(If(execute IsNot Nothing, Function(x) execute, Nothing), If(canExecute IsNot Nothing, Function(x) canExecute(), Nothing))
    End Sub
End Class